@@ -83,6 +83,22 @@ matrixParam<-function(x,p,matrix="symmetric",tol=0.000001){
         eigval<-eigen(A)
         Adecomp<-list(vectors=eigval$vectors, values=eigval$values, A=A, invectors=t(eigval$vectors))
     },
+    "lower"={
+        dim1<-p*(p-1)/2
+        A<-matrix(0,p,p)
+        A[lower.tri(A,diag=FALSE)] <- x[1:dim1]
+        diag(A) <- exp(x[(dim1+1):(dim1+p)])
+        eigval<-eigen(A)
+        Adecomp<-list(vectors=eigval$vectors, values=eigval$values, A=A, invectors=solve(eigval$vectors))
+    },
+    "upper"={
+        dim1<-p*(p-1)/2
+        A<-matrix(0,p,p)
+        A[upper.tri(A,diag=FALSE)] <- x[1:dim1]
+        diag(A) <- exp(x[(dim1+1):(dim1+p)])
+        eigval<-eigen(A)
+        Adecomp<-list(vectors=eigval$vectors, values=eigval$values, A=A, invectors=solve(eigval$vectors))
+    },
     "univariate"={
         Adecomp<-list(vectors=1, values=x, A=x, invectors=1)
     }
@@ -189,19 +205,23 @@ eval_polytom<-function(tree){
     nb.tip <- length(tree$tip.label)
     nb.node <- tree$Nnode
     if (nb.node != nb.tip - 1) {
-        cat("You can't use \"pic\" with polytomies, try instead \"rpf\",\"inverse\",\"pseudoinverse\" or \"sparse\"")
+        stop("You can't use \"pic\" with polytomies, try instead \"rpf\",\"inverse\",\"pseudoinverse\" or \"sparse\". Otherwise, first transform the tree using the \"multi2di\" function")
     }
 }
 
 # Estimate starting values for the variance matrix
 varBM<-function(tree,data,n,k){
+    if(any(is.na(data))){
+        res<-diag(0.1,k)
+        return(res)
+    }
     nb.tip <- length(tree$tip.label)
     nb.node <- tree$Nnode
     if (nb.node != nb.tip - 1) {
     tree <- multi2di(tree)
     }
     rate<-rep(0,k)
-    tree<-reorder(tree,"postorder")
+    tree<-reorder.phylo(tree,"postorder")
     value<-list(tree$edge.length)
     res<-.Call("PIC_gen", x=as.vector(as.matrix(data)), n=as.integer(k), Nnode=as.integer(tree$Nnode), nsp=as.integer(n), edge1=as.integer(tree$edge[,1]), edge2=as.integer(tree$edge[,2]), edgelength=value, times=1, rate=rate, Tmax=1, Model=as.integer(10), mu=1, sigma=1)
     return(res[[2]])
@@ -277,20 +297,27 @@ build.chol<-function(b,p){
 
 ##----------------------mvfit_likelihood--------------------------------------##
 
-loglik_mvmorph<-function(data,V=NULL,D=NULL,n,k,error=NULL,precalc=precalc,method, param=list(),ch=NULL,precalcMat=NULL,sizeD=NULL){
+loglik_mvmorph<-function(data,V=NULL,D=NULL,n,k,error=NULL,precalc=precalc,method, param=list(),ch=NULL,precalcMat=NULL,sizeD=NULL,NA_val=NULL,Indice_NA=NULL){
 
 data<-as.numeric(as.matrix(data))
+size<-k*n
+if(NA_val==TRUE){
+    V<-V[-Indice_NA,-Indice_NA]
+    D<-D[-Indice_NA,]
+    data<-data[-Indice_NA]
+    size<-length(data)
+}
+
 switch(method,
 
 "rpf"={
     if(is.null(error)!=TRUE){ ms<-1 }else{ ms<-0} # length of the error vector should be the same as the data vector
-    size<-k*n
     cholres<-.Call("Chol_RPF",V,D,data,as.integer(sizeD),as.integer(size),mserr=error,ismserr=as.integer(ms))
     beta<-pseudoinverse(cholres[[3]])%*%cholres[[4]]
     det<-cholres[[2]]
     residus=D%*%beta-data
     quad<-.Call("Chol_RPF_quadprod", cholres[[1]], residus, as.integer(size))
-    logl<--.5*quad-.5*as.numeric(det)-.5*(n*k*log(2*pi))
+    logl<--.5*quad-.5*as.numeric(det)-.5*(size*log(2*pi))
     results<-list(logl=logl,anc=beta)
 },
 "univarpf"={
@@ -301,7 +328,7 @@ switch(method,
     det<-cholres[[2]]
     residus=D%*%beta-data
     quad<-.Call("Chol_RPF_quadprod_column", cholres[[1]], residus, as.integer(size))
-    logl<--.5*quad-.5*as.numeric(det)-.5*(n*k*log(2*pi))
+    logl<--.5*quad-.5*as.numeric(det)-.5*(size*log(2*pi))
     results<-list(logl=logl,anc=beta)
 },
 "sparse"={
@@ -314,9 +341,9 @@ switch(method,
         }
     }
     if(!is.null(precalc)){
-    U<-update(precalc$ch,precalc$V)
+    U<-update.spam.chol.NgPeyton(precalc$ch,precalc$V)
     }else{
-    U<-update(ch,precalcMat)
+    U<-update.spam.chol.NgPeyton(ch,precalcMat)
     }
     vec<-forwardsolve(U,data)
     xx<-forwardsolve(U,D)
@@ -337,7 +364,7 @@ switch(method,
     beta<-pseudoinverse(t(D)%*%inv%*%D)%*%t(D)%*%inv%*%data
     DET<-determinant(V, logarithm=TRUE)
     res<-D%*%beta-data
-    logl<--.5*(t(res)%*%inv%*%(res))-.5*as.numeric(DET$modulus)-.5*(n*k*log(2*pi))
+    logl<--.5*(t(res)%*%inv%*%(res))-.5*as.numeric(DET$modulus)-.5*(size*log(2*pi))
     results<-list(logl=logl,anc=beta)
 },
 "inverse"={
@@ -349,7 +376,7 @@ switch(method,
     beta<-solve(t(D)%*%inv%*%D)%*%t(D)%*%inv%*%data
     DET<-determinant(V, logarithm=TRUE)
     res<-D%*%beta-data
-    logl<--.5*(t(res)%*%inv%*%(res))-.5*as.numeric(DET$modulus)-.5*(n*k*log(2*pi))
+    logl<--.5*(t(res)%*%inv%*%(res))-.5*as.numeric(DET$modulus)-.5*(size*log(2*pi))
     results<-list(logl=logl,anc=beta)
 })
 
@@ -555,4 +582,4 @@ halflife.mvmorph<-function(object){
     }else{
         warning("The phylogenetic half-life is computed only for Ornstein-Uhlenbeck models.","\n")  
     }
-}
+}
\ No newline at end of file
